@@ -7,6 +7,11 @@ import quopri, random, re, rfc822, stat, sys, time
 import socket, errno
 from traceback import format_exc
 
+try:
+    from OpenSSL.SSL import SysCallError
+except ImportError:
+    SysCallError = None
+
 from roundup import roundupdb, date, hyperdb, password
 from roundup.cgi import templating, cgitb, TranslationService
 from roundup.cgi.actions import *
@@ -180,7 +185,7 @@ class Session:
 
             if 'set_cookie' is True, set cookie with 'expire' seconds lifetime
             if 'expire' is None - session will be closed with the browser
-             
+
             XXX the session can be purged within a week even if a cookie
                 lifetime is longer
         """
@@ -210,7 +215,7 @@ class Client:
 
     During the processing of a request, the following attributes are used:
 
-    - "db" 
+    - "db"
     - "error_message" holds a list of error messages
     - "ok_message" holds a list of OK messages
     - "session" is deprecated in favor of session_api (XXX remove)
@@ -356,7 +361,7 @@ class Client:
         """ Wrap the real main in a try/finally so we always close off the db.
         """
         try:
-            if self.env.get('CONTENT_TYPE') == 'text/xml':
+            if self.env.get('CONTENT_TYPE') == 'text/xml' and self.path == 'xmlrpc':
                 self.handle_xmlrpc()
             else:
                 self.inner_main()
@@ -393,7 +398,7 @@ class Client:
         self.setHeader("Content-Type", "text/xml")
         self.setHeader("Content-Length", str(len(output)))
         self.write(output)
-        
+
     def inner_main(self):
         """Process a request.
 
@@ -483,7 +488,10 @@ class Client:
                 self.serve_static_file(str(file))
             except IOError:
                 # IOErrors here are due to the client disconnecting before
-                # recieving the reply.
+                # receiving the reply.
+                pass
+            except SysCallError:
+                # OpenSSL.SSL.SysCallError is similar to IOError above
                 pass
 
         except SeriousError, message:
@@ -531,6 +539,15 @@ class Client:
         except FormError, e:
             self.error_message.append(self._('Form Error: ') + str(e))
             self.write_html(self.renderContext())
+        except IOError:
+            # IOErrors here are due to the client disconnecting before
+            # receiving the reply.
+            # may happen during write_html and serve_file, too.
+            pass
+        except SysCallError:
+            # OpenSSL.SSL.SysCallError is similar to IOError above
+            # may happen during write_html and serve_file, too.
+            pass
         except:
             # Something has gone badly wrong.  Therefore, we should
             # make sure that the response code indicates failure.
@@ -689,6 +706,10 @@ class Client:
                         pass
                     username, password = decoded.split(':', 1)
                     try:
+                        # Current user may not be None, otherwise
+                        # instatiation of the login action will fail.
+                        # So we set the user to anonymous first.
+                        self.make_user_anonymous()
                         login = self.get_action_class('login')(self)
                         login.verifyLogin(username, password)
                     except LoginError, err:
@@ -780,7 +801,7 @@ class Client:
                 # The old session API refers to the closed database;
                 # we can no longer use it.
                 self.session_api = Session(self)
- 
+
 
     def determine_context(self, dre=re.compile(r'([^\d]+)0*(\d+)')):
         """Determine the context of this page from the URL:
@@ -900,7 +921,7 @@ class Client:
         except KeyError:
             # The classname was not valid.
             raise NotFound(str(designator))
-            
+
         # perform the Anonymous user access check
         self.check_anonymous_access()
 
@@ -951,7 +972,7 @@ class Client:
                 pass
         if not filename:
             content = klass.get(nodeid, 'content')
-        
+
         lmt = klass.get(nodeid, 'activity').timestamp()
 
         self._serve_file(lmt, mime_type, content, filename)
@@ -1033,10 +1054,10 @@ class Client:
         part = MIMEText(txt)
         message.attach(part)
         self.mailer.smtp_send(to, message.as_string())
-    
+
     def renderFrontPage(self, message):
         """Return the front page of the tracker."""
-    
+
         self.classname = self.nodeid = None
         self.template = ''
         self.error_message.append(message)
@@ -1097,7 +1118,7 @@ class Client:
                 # receive an error message, and the adminstrator will
                 # receive a traceback, albeit with less information
                 # than the one we tried to generate above.
-                raise exc_info[0](exc_info[1]).with_traceback(exc_info[2])
+                raise exc_info[0], exc_info[1], exc_info[2]
 
     # these are the actions that are available
     actions = (
@@ -1254,7 +1275,7 @@ class Client:
         elements = [self.http_strip(e) for e in elements]
         # Remove any now-empty elements.
         return [e for e in elements if e]
-        
+
     def handle_range_header(self, length, etag):
         """Handle the 'Range' and 'If-Range' headers.
 
@@ -1291,7 +1312,7 @@ class Client:
         if_range = self.env.get("HTTP_IF_RANGE")
         if if_range:
             # The grammar for the If-Range header is:
-            # 
+            #
             #   If-Range = "If-Range" ":" ( entity-tag | HTTP-date )
             #   entity-tag = [ weak ] opaque-tag
             #   weak = "W/"
@@ -1396,7 +1417,7 @@ class Client:
         length = stat_info[stat.ST_SIZE]
         # Assume we will return the entire file.
         offset = 0
-        # If the headers have not already been finalized, 
+        # If the headers have not already been finalized,
         if not self.headers_done:
             # RFC 2616 14.19: ETag
             #
